"""
Available tables and currencies.

Data taken from /api/exchangerates/tables/{a,b,c}/ API calls.
"""

import json


class NBPCurrency(object):
    """
    Helper class holding basic currency info.

    This class shouldn't be used outside of ``currencies`` from this
    submodule.
    """

    def __init__(self, code, name, tables):
        r"""
        NBPCurrency init.

        :param code:
            Currency code in accordance to ISO 4217 standard. Argument is not
            validated, you are free to use any string.

        :param name:
            Currency full name.

        :param tables:
            Container with tables available to given currency.
        """
        self.code = code
        self.name = name
        self.tables = set(tables)

    def __repr__(self):
        """Return repr(self)."""
        return "{cls_name}({name}, code={code}, tables={tables})".format(
            cls_name=self.__class__.__name__,
            name=self.name,
            code=self.code,
            tables=str(self.tables)
        )

    def __str__(self):
        """Return str(self)."""
        return self.name


#: Available currencies
currencies = {
    'AED': {'name': 'United Arab Emirates dirham',
            'tables': ('B',)},
    'AFN': {'name': 'Afghan afghani',
            'tables': ('B',)},
    'ALL': {'name': 'Albanian lek',
            'tables': ('B',)},
    'AMD': {'name': 'Armenian dram',
            'tables': ('B',)},
    'ANG': {'name': 'Netherlands Antillean guilder',
            'tables': ('B',)},
    'AOA': {'name': 'Angolan kwanza',
            'tables': ('B',)},
    'ARS': {'name': 'Argentine peso',
            'tables': ('B',)},
    'AUD': {'name': 'Australian dollar',
            'tables': ('A', 'C')},
    'AWG': {'name': 'Aruban florin',
            'tables': ('B',)},
    'AZN': {'name': 'Azerbaijani manat',
            'tables': ('B',)},
    'BAM': {'name': 'Bosnia and Herzegovina convertible mark',
            'tables': ('B',)},
    'BBD': {'name': 'Barbados dollar',
            'tables': ('B',)},
    'BDT': {'name': 'Bangladeshi taka',
            'tables': ('B',)},
    'BGN': {'name': 'Bulgarian lev',
            'tables': ('A',)},
    'BHD': {'name': 'Bahraini dinar',
            'tables': ('B',)},
    'BIF': {'name': 'Burundian franc',
            'tables': ('B',)},
    'BND': {'name': 'Brunei dollar',
            'tables': ('B',)},
    'BOB': {'name': 'Boliviano',
            'tables': ('B',)},
    'BRL': {'name': 'Brazilian real',
            'tables': ('A',)},
    'BSD': {'name': 'Bahamian dollar',
            'tables': ('B',)},
    'BWP': {'name': 'Botswana pula',
            'tables': ('B',)},
    'BYN': {'name': 'Belarusian ruble',
            'tables': ('B',)},
    'BZD': {'name': 'Belize dollar',
            'tables': ('B',)},
    'CAD': {'name': 'Canadian dollar',
            'tables': ('A', 'C')},
    'CDF': {'name': 'Congolese franc',
            'tables': ('B',)},
    'CHF': {'name': 'Swiss franc',
            'tables': ('A', 'C')},
    'CLP': {'name': 'Chilean peso',
            'tables': ('A',)},
    'CNY': {'name': 'Chinese yuan',
            'tables': ('A',)},
    'COP': {'name': 'Colombian peso',
            'tables': ('B',)},
    'CRC': {'name': 'Costa Rican colon',
            'tables': ('B',)},
    'CUP': {'name': 'Cuban peso',
            'tables': ('B',)},
    'CVE': {'name': 'Cape Verde escudo',
            'tables': ('B',)},
    'CZK': {'name': 'Czech koruna',
            'tables': ('A', 'C')},
    'DJF': {'name': 'Djiboutian franc',
            'tables': ('B',)},
    'DKK': {'name': 'Danish krone',
            'tables': ('A', 'C')},
    'DOP': {'name': 'Dominican peso',
            'tables': ('B',)},
    'DZD': {'name': 'Algerian dinar',
            'tables': ('B',)},
    'EGP': {'name': 'Egyptian pound',
            'tables': ('B',)},
    'ERN': {'name': 'Eritrean nakfa',
            'tables': ('B',)},
    'ETB': {'name': 'Ethiopian birr',
            'tables': ('B',)},
    'EUR': {'name': 'Euro',
            'tables': ('A', 'C')},
    'FJD': {'name': 'Fiji dollar',
            'tables': ('B',)},
    'GBP': {'name': 'Pound sterling',
            'tables': ('A', 'C')},
    'GEL': {'name': 'Georgian lari',
            'tables': ('B',)},
    'GHS': {'name': 'Ghanaian cedi',
            'tables': ('B',)},
    'GIP': {'name': 'Gibraltar pound',
            'tables': ('B',)},
    'GMD': {'name': 'Gambian dalasi',
            'tables': ('B',)},
    'GNF': {'name': 'Guinean franc',
            'tables': ('B',)},
    'GTQ': {'name': 'Guatemalan quetzal',
            'tables': ('B',)},
    'GYD': {'name': 'Guyanese dollar',
            'tables': ('B',)},
    'HKD': {'name': 'Hong Kong dollar',
            'tables': ('A',)},
    'HNL': {'name': 'Honduran lempira',
            'tables': ('B',)},
    'HRK': {'name': 'Croatian kuna',
            'tables': ('A',)},
    'HTG': {'name': 'Haitian gourde',
            'tables': ('B',)},
    'HUF': {'name': 'Hungarian forint',
            'tables': ('A', 'C')},
    'IDR': {'name': 'Indonesian rupiah',
            'tables': ('A',)},
    'ILS': {'name': 'Israeli new shekel',
            'tables': ('A',)},
    'INR': {'name': 'Indian rupee',
            'tables': ('A',)},
    'IQD': {'name': 'Iraqi dinar',
            'tables': ('B',)},
    'IRR': {'name': 'Iranian rial',
            'tables': ('B',)},
    'ISK': {'name': 'Icelandic króna',
            'tables': ('A',)},
    'JMD': {'name': 'Jamaican dollar',
            'tables': ('B',)},
    'JOD': {'name': 'Jordanian dinar',
            'tables': ('B',)},
    'JPY': {'name': 'Japanese yen',
            'tables': ('A', 'C')},
    'KES': {'name': 'Kenyan shilling',
            'tables': ('B',)},
    'KGS': {'name': 'Kyrgyzstani som',
            'tables': ('B',)},
    'KHR': {'name': 'Cambodian riel',
            'tables': ('B',)},
    'KMF': {'name': 'Comoro franc',
            'tables': ('B',)},
    'KRW': {'name': 'South Korean won',
            'tables': ('A',)},
    'KWD': {'name': 'Kuwaiti dinar',
            'tables': ('B',)},
    'KZT': {'name': 'Kazakhstani tenge',
            'tables': ('B',)},
    'LAK': {'name': 'Lao kip',
            'tables': ('B',)},
    'LBP': {'name': 'Lebanese pound',
            'tables': ('B',)},
    'LKR': {'name': 'Sri Lankan rupee',
            'tables': ('B',)},
    'LRD': {'name': 'Liberian dollar',
            'tables': ('B',)},
    'LSL': {'name': 'Lesotho loti',
            'tables': ('B',)},
    'LYD': {'name': 'Libyan dinar',
            'tables': ('B',)},
    'MAD': {'name': 'Moroccan dirham',
            'tables': ('B',)},
    'MDL': {'name': 'Moldovan leu',
            'tables': ('B',)},
    'MGA': {'name': 'Malagasy ariary',
            'tables': ('B',)},
    'MKD': {'name': 'Macedonian denar',
            'tables': ('B',)},
    'MMK': {'name': 'Myanmar kyat',
            'tables': ('B',)},
    'MNT': {'name': 'Mongolian tögrög',
            'tables': ('B',)},
    'MOP': {'name': 'Macanese pataca',
            'tables': ('B',)},
    'MRO': {'name': 'Mauritanian ouguiya',
            'tables': ('B',)},
    'MUR': {'name': 'Mauritian rupee',
            'tables': ('B',)},
    'MVR': {'name': 'Maldivian rufiyaa',
            'tables': ('B',)},
    'MWK': {'name': 'Malawian kwacha',
            'tables': ('B',)},
    'MXN': {'name': 'Mexican peso',
            'tables': ('A',)},
    'MYR': {'name': 'Malaysian ringgit',
            'tables': ('A',)},
    'MZN': {'name': 'Mozambican metical',
            'tables': ('B',)},
    'NAD': {'name': 'Namibian dollar',
            'tables': ('B',)},
    'NGN': {'name': 'Nigerian naira',
            'tables': ('B',)},
    'NIO': {'name': 'Nicaraguan córdoba',
            'tables': ('B',)},
    'NOK': {'name': 'Norwegian krone',
            'tables': ('A', 'C')},
    'NPR': {'name': 'Nepalese rupee',
            'tables': ('B',)},
    'NZD': {'name': 'New Zealand dollar',
            'tables': ('A',)},
    'OMR': {'name': 'Omani rial',
            'tables': ('B',)},
    'PAB': {'name': 'Panamanian balboa',
            'tables': ('B',)},
    'PEN': {'name': 'Peruvian Sol',
            'tables': ('B',)},
    'PGK': {'name': 'Papua New Guinean kina',
            'tables': ('B',)},
    'PHP': {'name': 'Philippine piso',
            'tables': ('A',)},
    'PKR': {'name': 'Pakistani rupee',
            'tables': ('B',)},
    'PYG': {'name': 'Paraguayan guaraní',
            'tables': ('B',)},
    'QAR': {'name': 'Qatari riyal',
            'tables': ('B',)},
    'RON': {'name': 'Romanian leu',
            'tables': ('A',)},
    'RSD': {'name': 'Serbian dinar',
            'tables': ('B',)},
    'RUB': {'name': 'Russian ruble',
            'tables': ('A',)},
    'RWF': {'name': 'Rwandan franc',
            'tables': ('B',)},
    'SAR': {'name': 'Saudi riyal',
            'tables': ('B',)},
    'SBD': {'name': 'Solomon Islands dollar',
            'tables': ('B',)},
    'SCR': {'name': 'Seychelles rupee',
            'tables': ('B',)},
    'SDG': {'name': 'Sudanese pound',
            'tables': ('B',)},
    'SEK': {'name': 'Swedish krona/kronor',
            'tables': ('A', 'C')},
    'SGD': {'name': 'Singapore dollar',
            'tables': ('A',)},
    'SLL': {'name': 'Sierra Leonean leone',
            'tables': ('B',)},
    'SOS': {'name': 'Somali shilling',
            'tables': ('B',)},
    'SRD': {'name': 'Surinamese dollar',
            'tables': ('B',)},
    'SSP': {'name': 'South Sudanese pound',
            'tables': ('B',)},
    'STD': {'name': 'São Tomé and Príncipe dobra',
            'tables': ('B',)},
    'SVC': {'name': 'Salvadoran colón',
            'tables': ('B',)},
    'SYP': {'name': 'Syrian pound',
            'tables': ('B',)},
    'SZL': {'name': 'Swazi lilangeni',
            'tables': ('B',)},
    'THB': {'name': 'Thai baht',
            'tables': ('A',)},
    'TJS': {'name': 'Tajikistani somoni',
            'tables': ('B',)},
    'TMT': {'name': 'Turkmenistan manat',
            'tables': ('B',)},
    'TND': {'name': 'Tunisian dinar',
            'tables': ('B',)},
    'TOP': {'name': 'Tongan paʻanga',
            'tables': ('B',)},
    'TRY': {'name': 'Turkish lira',
            'tables': ('A',)},
    'TTD': {'name': 'Trinidad and Tobago dollar',
            'tables': ('B',)},
    'TWD': {'name': 'New Taiwan dollar',
            'tables': ('B',)},
    'TZS': {'name': 'Tanzanian shilling',
            'tables': ('B',)},
    'UAH': {'name': 'Ukrainian hryvnia',
            'tables': ('A',)},
    'UGX': {'name': 'Ugandan shilling',
            'tables': ('B',)},
    'USD': {'name': 'United States dollar',
            'tables': ('A', 'C')},
    'UYU': {'name': 'Uruguayan peso',
            'tables': ('B',)},
    'UZS': {'name': 'Uzbekistan som',
            'tables': ('B',)},
    'VEF': {'name': 'Venezuelan bolívar',
            'tables': ('B',)},
    'VND': {'name': 'Vietnamese đồng',
            'tables': ('B',)},
    'VUV': {'name': 'Vanuatu vatu',
            'tables': ('B',)},
    'WST': {'name': 'Samoan tala',
            'tables': ('B',)},
    'XAF': {'name': 'CFA franc BEAC',
            'tables': ('B',)},
    'XCD': {'name': 'East Caribbean dollar',
            'tables': ('B',)},
    'XDR': {'name': 'Special drawing rights',
            'tables': ('A', 'C')},
    'XOF': {'name': 'CFA franc BCEAO',
            'tables': ('B',)},
    'XPF': {'name': 'CFP franc (franc Pacifique)',
            'tables': ('B',)},
    'YER': {'name': 'Yemeni rial',
            'tables': ('B',)},
    'ZAR': {'name': 'South African rand',
            'tables': ('A',)},
    'ZMW': {'name': 'Zambian kwacha',
            'tables': ('B',)},
}

currencies = {
    code: NBPCurrency(code=code, **items)
    for code, items in currencies.items()
}
